/*******************************************************************************
* File Name: main.c
*
* Version: 1.0
*
* Description:
* This project demonstrates a 24 Bit Low Pass Filter Operation. For this, we are using
* ADC configured for 18 bit resolution and DAC configured for 12 bit resolution
*
* Firmware Dependency:
* PSoC Creator 3.3 SP1 and above
*
* Hardware Dependency:
* CY8CKIT-050 PSoC 5LP Development Kit
*
********************************************************************************
* Copyright 2014-2015, Cypress Semiconductor Corporation.  All rights reserved.
* This software is owned by Cypress Semiconductor Corporation and is protected
* by and subject to worldwide patent and copyright laws and treaties.
* Therefore, you may use this software only as provided in the license agreement
* accompanying the software package from which you obtained this software.
* CYPRESS AND ITS SUPPLIERS MAKE NO WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
* WITH REGARD TO THIS SOFTWARE, INCLUDING, BUT NOT LIMITED TO, NONINFRINGEMENT,
* IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
*******************************************************************************/

#include <project.h>

/* Necessary defines for DMA Configuration. Request Per Burst is set to 1 as ADC
End Of Conversion triggers DMA. Three Bytes are transferred per burst to write into 
all the three Staging Registers. */

#define REQUEST_PER_BURST        (1u)
#define BYTES_PER_BURST          (3u)
#define UPPER_SRC_ADDRESS        CYDEV_PERIPH_BASE
#define UPPER_DEST_ADDRESS       CYDEV_PERIPH_BASE

/* Maximum Value for 12 bit VDAC */
#define VDAC_MAX				 0xFF0

/* General constants */
#define MASK_12BIT              (0xFFFu)
#define SHIFT_THREE             (0x03u)  

#define FiltA

/* Function to configure DMA Channel */
void DMA_Config(void);

/* Interrpt service routine to read the filterd data */
CY_ISR_PROTO(Filter_Done);

/* Variable to Hold the Filter Output */
int32 Filter_Out;

/*******************************************************************************
* Function Name: main
********************************************************************************
*
* Summary:
*  Starts all the components and configures DMA. The function also writes the Filtered 
*  Output to VDAC once valid data is present in Filter's Holding Register
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/

int main()
{
    /* Start all components used on schematic */
	ADC_DelSig_Start();
    ADC_DelSig_StartConvert();
	ADC_DelSig_IRQ_Start();
    VDAC_Start();
	Filter_Start();
    isr_Filter_StartEx(Filter_Done);
	
	/* Configure the DMA */
	DMA_Config();

	/* Set the Filter Coherency to High Byte */
	Filter_SetCoherency(Filter_CHANNEL_A, Filter_KEY_HIGH);
	Filter_SetCoherency(Filter_CHANNEL_B, Filter_KEY_HIGH);	
    /* Enable Global Interrupts */
    CYGlobalIntEnable;

    for(;;)
    {	
		
    }

} /* End of main */

/*******************************************************************************
* Interrupt
********************************************************************************
* Interrupt generated on Filter sample-ready. Interrupt handle:Filter_Done
*
* Summary:
*  The interrupt performs following functions:
*   1: Reads the 16 bit MSB Aligned Filter Output for Filter Channel A
*   2: Writes the most significant 12 bits of filterd data to VDAC 

*******************************************************************************/
CY_ISR(Filter_Done)
{
    /* Read the 16 bit MSB Aligned Filter output */
#ifdef FiltA
   	Filter_Out = ((Filter_Read16(Filter_CHANNEL_A) >> SHIFT_THREE) & MASK_12BIT) ;
#else
    Filter_Out = ((Filter_Read16(Filter_CHANNEL_B) >> SHIFT_THREE) & MASK_12BIT) ;
#endif	
	/* Saturate the Output value if it exceeds maximum VDAC value */
	if(Filter_Out > VDAC_MAX)
		Filter_Out = VDAC_MAX;
	
	/* Write the value to VDAC */
	VDAC_SetValue(Filter_Out);
}
/*******************************************************************************
* Function Name: DMA_Config
********************************************************************************
*
* Summary:
*  Initializes and sets up DMA for use (generated by DMA Wizard)
*
* Parameters:
*  None.
*
* Return:
*  None.
*
*******************************************************************************/
void DMA_Config(void)
{
    /* Declare variable to hold the handle for DMA channel */
    uint8 channelHandle;

    /* Declare DMA Transaction Descriptor for memory transfer into
     * Filter Channel. */
    uint8 tdChanA;

    /* Configure the DMA to Transfer the data in 1 burst with individual trigger
     * for each burst.*/
    channelHandle = DMA_DmaInitialize(BYTES_PER_BURST, REQUEST_PER_BURST,
                                        HI16(UPPER_SRC_ADDRESS), HI16(UPPER_DEST_ADDRESS));

    /* This function allocates a TD for use with an initialized DMA channel */
    tdChanA = CyDmaTdAllocate();

	/* Source and Destination address increments are needed as we are using 3 byte transfers
	but Spoke Width is 16 bit */
    CyDmaTdSetConfiguration(tdChanA, 3u, tdChanA, TD_INC_SRC_ADR | TD_INC_DST_ADR);

    /* Set the source address as ADC_DelSig and the destination as
     * Filter Channel A.*/
#ifdef FiltA    
    CyDmaTdSetAddress(tdChanA, LO16((uint32)ADC_DelSig_DEC_SAMP_PTR), LO16((uint32)Filter_STAGEA_PTR));
#else
    CyDmaTdSetAddress(tdChanA, LO16((uint32)ADC_DelSig_DEC_SAMP_PTR), LO16((uint32)Filter_STAGEB_PTR));
#endif
    /* Set tdChanA to be the initial TD associated with channelHandle */
    CyDmaChSetInitialTd(channelHandle, tdChanA);

    /* Enable the DMA channel represented by channelHandle and preserve the TD */
    CyDmaChEnable(channelHandle, 1u);
}

/* [] END OF FILE */
